#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['aio_wx_widgets',
 'aio_wx_widgets.containers',
 'aio_wx_widgets.core',
 'aio_wx_widgets.panels',
 'aio_wx_widgets.widgets']

package_data = \
{'': ['*']}

install_requires = \
['wxasync', 'wxpython', 'pydantic', 'events']

extras_require = \
{'dev': ['pylint', 'nox', 'pytest-cov', 'coverage[toml]>=5.0.3', 'pre-commit'],
 "dev:python_version=='3.7'": ['typing-extensions'],
 'test': ['pytest', 'pytest-asyncio', 'pytest-mock', 'mypy']}

setup(name='aio_wx_widgets',
      version='1.0.4',
      description='aio_wx_widgets',
      author='Sander Teunissen',
      author_email=None,
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.7',
     )
